; XYSCALE.LSP

; Allows different X and Y scale factor for scaling entities
; Copyright 1988 Alacrity
; Public Domain, Please distribute freely
;
; ==[ Introduction ]==
;
; XYSCALE works much the same way as the SCALE command.  But it allows the user
; to specify different X and Y scales. This allows some useful transformations.
; Like elongating entities in the X axis without changing the Y axis.
;
; This documentation is not very thorough.  It is assumed that you already know
; a fair amount about AutoCAD. It helps to have already used the SCALE command.
;
; ==[ Program Usage ]==
;
; Before you can use XYSCALE.LSP you will need to load it:
;
;   Command: (load "XYSCALE") [ENTER]
;
; This is what will appear when XYSCALE is properly loaded:
;
;   Copyright (c)1988 Alacrity
;   C:XYSCALE
;
; The following is an example usage:
;
;   Command: XYSCALE [ENTER]
;   Select objects: <make selections>
;   Base point: <select point>
;   X scale factor/Reference: <x-scale> [ENTER]
;   Y scale factor/Reference: R [ENTER]
;   Reference length: 1.0 [ENTER]
;   New length: 1.125 [ENTER]
; 
; ==[ Program notes ]==
;
; - XYSCALE only changes the physical dimensions of POLYLINEs and LINEs.  For
;   other entities, XYSCALE simply moves the insertion point (i.e. center of
;   a circle) in relation to the base point.
;
; - XYSCALE does not treat Curved/Spline POLYLINEs or POLYLINEs with arced
;   segments properly.  Only use on straight POLYINEs.
;
; - XYSCALE does not treat SOLIDs, TRACEs, 3DLINEs or 3DFACEs properly.  Do
;   not include said objects in selection set.
;
; - Negative scale factors effectively perform a mirror operation.
;
; ==[ Author's notes ]==
;
; I created XYSCALE out of frustration.  It would be a neat capability to
; EXPLODE blocks that have different X and Y scales.  But since this is not
; possible, I created a work around.
;
; My initial use was for profiles in Civil work.  Profiles are often drawn with
; different X and Y scales.  It is tedious to compute the locations of line
; endpoints to take into account the X/Y scale.  With XYSCALE, I simply draw
; the profile to scale and then transform it afterwards.  No more headache.
;
; Please use this routine however you see fit.  There is no warantee written or
; otherwise.  XYSCALE.LSP may not be distributed with any other software
; without the author's permission.  Also, please leave these comments and
; copyright notice intact.  Especially if giving this routine to others.
;
; Ideas for improvements and such are welcome.  If you make an improvement of
; your own, I would enjoy hearing about it.  Please please please report bugs.
;
; Jason Osgood
; Alacrity
; 12405 SE 25th St
; Bellevue, WA 98005
;
; Alacrity BBS: (206)643-5477
; Voice: (206)746-0680
; CompuServe: 73417,1756

(princ "\nCopyright (c)1988 Alacrity\n")

(defun C:XYSCALE (/ trans etype refer ss basepnt x0 y0 pnt pnt1 pnt2 i
                    xscale yscale entname entlist pntlst1 pntlst2 num)
  ;-------------------------------------------------------------
  ; Translate coordinates using base point and X/Y scale factor
  ;-------------------------------------------------------------
  (defun trans (pnt / x y)
    (list (+ (* (- (car  pnt) x0) xscale) x0)
          (+ (* (- (cadr pnt) y0) yscale) y0)
    )
  )

  ;--------------------
  ; Return entity type
  ;--------------------
  (defun etype (entlist)
    (cdr (assoc 0 entlist))
  )

  ;-----------------------
  ; Enter reference scale
  ;-----------------------
  (defun refer (/ d1 d2)
    (initget 7)
    (setq d1 (getdist "\nReference length: "))
    (initget 7)
    (setq d2 (getdist "\nNew length: "))
    (/ d1 d2)
  )

  ;----------------
  ; Get User Input
  ;----------------
  (setq ss (ssget)
        basepnt (getpoint "\nBase point: ")
  )
  (initget 3 "Refer")
  (setq xscale (getdist "\nX scale factor /Reference: "))
  (if (= xscale "Refer")
    (setq xscale (refer))
  )
  (initget 3 "Refer")
  (setq yscale (getdist "\nY scale factor /Reference: "))
  (if (= yscale "Refer")
    (setq yscale (refer))
  )
  (setq x0 (car basepnt)
        y0 (cadr basepnt)
        num (sslength ss)
        i 0
  )

  ;----------------------
  ; X/Y Scale everything
  ;----------------------
  (while (< i num)
    (setq entname (ssname ss i)
          entlist (entget entname)
    )
    (cond
      ((= (etype entlist) "LINE")
        (progn
          (setq pnt1 (cdr (setq pntlst1 (assoc 10 entlist)))
                pnt2 (cdr (setq pntlst2 (assoc 11 entlist)))
                pnt1 (trans pnt1)
                pnt2 (trans pnt2)
                entlist (subst (cons 10 pnt1) pntlst1 entlist)
                entlist (subst (cons 11 pnt2) pntlst2 entlist)
           )
           (entmod entlist)
         )
      )
      ((= (etype entlist) "POLYLINE")
        (progn
          (while (/= (etype entlist) "SEQEND")
            (if (= (etype entlist) "VERTEX")
              (progn
                (setq pnt1 (cdr (setq pntlst1 (assoc 10 entlist)))
                      pnt1 (trans pnt1)
                      entlist (subst (cons 10 pnt1) pntlst1 entlist)
                )
                (entmod entlist)
              )
            )
            (setq entlist (entget (setq entname (entnext entname))))
          )
        )
      )
      (T
        (progn
          (setq pnt1 (cdr (setq pntlst1 (assoc 10 entlist)))
                pnt1 (trans pnt1)
                entlist (subst (cons 10 pnt1) pntlst1 entlist)
          )
          (entmod entlist)
        )
      )
    )
    (setq i (1+ i))
  )
  (princ)
)

; End of File